@@ -59,15 +59,24 @@ progress_bar <- function(i,n,len=30){
 #' Returns the file name and type for MCNR function
 #' 
 #' @param family family object
+#' @param cmdstan Logical indicating whether cmdstan is being used and the function will return the filename
 #' @return list with filename and type
-mcnr_family <- function(family){
-  f1 <- family[[1]]
+mcnr_family <- function(family, cmdstan){
+  f1 <- tolower(family[[1]])
   link <- family[[2]]
   gaussian_list <- c("identity")
   binomial_list <- c("logit","log","identity","probit")
+  bernoulli_list <- c("logit","log","identity","probit")
   poisson_list <- c("log")
+  gamma_list <- c("identity","inverse","log")
+  beta_list <- c("logit")
   type <- which(get(paste0(f1,"_list"))==link)
-  return(list(file = paste0("mcml_",f1,".stan"),type=type))
+  if(length(type)==0)stop("link not supported for this family")
+  if(cmdstan){
+    return(list(file = paste0("mcml_",f1,".stan"),type=type))
+  } else {
+    return(list(file = paste0("mcml_",f1),type=type))#,".stan"
+  }
 }
 
 #' Data for model tests
