@@ -217,10 +217,113 @@ inline Eigen::VectorXd detadmu(const Eigen::VectorXd& xb,
     break;
     
   }
-  
   return wdiag;
 }
 
+// inline double log_likelihood_derivative(const double mu, 
+//                                         const double y,
+//                                         int order, 
+//                                         const Fam family,
+//                                         double var_par = 1.0)
+// {
+//   if(order < 1 || order > 2)throw std::runtime_error("Only orders 1 and 2 supported for derivatives");
+//   double result;
+//   
+//   if(order == 1){
+//     switch (family) {
+//     case Fam::gaussian:
+//       result = (y - mu)/var_par;
+//       break;
+//     case Fam::bernoulli: 
+//       result = y/mu - (1-y)/(1-mu);
+//       break;
+//     case Fam::binomial:
+//       result = y/mu - (var_par-y)/(1-mu);
+//       break;
+//     case Fam::poisson:
+//       result = y/mu - 1;
+//       break;
+//     case Fam::gamma:
+//       result = 0;
+//       break;
+//     case Fam::beta:
+//       result = 0;
+//       break;
+//     }
+//   } else {
+//     switch (family) {
+//     case Fam::gaussian:
+//       result = -1.0/var_par;
+//       break;
+//     case Fam::bernoulli: 
+//       result = -1.0*y/(mu*mu) - (1-y)/((1-mu)*(1-mu));
+//       break;
+//     case Fam::binomial:
+//       result = -1.0*y/(mu*mu) - (var_par-y)/((1-mu)*(1-mu));
+//       break;
+//     case Fam::poisson:
+//       result = -1.0*y/(mu*mu);
+//       break;
+//     case Fam::gamma:
+//       result = 0;
+//       break;
+//     case Fam::beta:
+//       result = 0;
+//       break;
+//     }
+//   }
+//   return result;
+// }
+// 
+// inline double link_derivative(const double eta, 
+//                               int order, 
+//                               const Link link)
+// {
+//   if(order < 1 || order > 2)throw std::runtime_error("Only orders 1 and 2 supported for derivatives");
+//   double result;
+//   
+//   if(order == 1){
+//     switch (link) {
+//     case Link::identity:
+//       result = 1.0;
+//       break;
+//     case Link::loglink:
+//       result = exp(eta);
+//       break;
+//     case Link::logit:
+//       {
+//         double exp_eta = exp(eta);
+//         result = exp_eta/((1+exp_eta)*(1+exp_eta));
+//         break;
+//       }
+//     default:
+//       result = 0;
+//       break;
+//     }
+//   } else {
+//     switch (link) {
+//     case Link::identity:
+//       result = 0.0;
+//       break;
+//     case Link::loglink:
+//       result = exp(eta);
+//       break;
+//     case Link::logit:
+//     {
+//       double exp_eta = exp(eta);
+//       result = exp_eta*(1+exp_eta)/((1+exp_eta)*(1+exp_eta)*(1+exp_eta));
+//       break;
+//     }
+//     default:
+//       result = 0;
+//       break;
+//     }
+//   }
+//   return result;
+// }
+
+
+
 inline Eigen::VectorXd attenuted_xb(const Eigen::VectorXd& xb,
                                     const Eigen::MatrixXd& Z,
                                     const Eigen::MatrixXd& D,
@@ -259,7 +362,6 @@ inline Eigen::VectorXd attenuted_xb(const Eigen::VectorXd& xb,
   return xbnew.matrix();
 }
 
-// is this function used anywhere? mark for deletion?
 inline Eigen::VectorXd marginal_var(const Eigen::VectorXd& mu,
                                     const Fam family,
                                     double var_par = 1.0){
@@ -451,7 +553,7 @@ inline double logdet(const Eigen::MatrixXd& M) {
   double ld = 0;
   Eigen::LLT<Eigen::MatrixXd> chol(M);
   auto& U = chol.matrixL();
-  for (unsigned i = 0; i < M.rows(); ++i)
+  for (int i = 0; i < M.rows(); ++i)
     ld += log(U(i,i));
   ld *= 2;
   return ld;
